import { Component, ViewChild } from '@angular/core';
import { Platform , Nav, AlertController } from 'ionic-angular';
import { StatusBar, Splashscreen } from 'ionic-native';

import { TabsPage } from '../pages/tabs/tabs';

import { Api } from '../providers/api';
import { Storage } from '@ionic/storage';

import { LocalNotifications } from 'ionic-native';
import { BackgroundMode } from 'ionic-native';
import { InAppBrowser } from 'ionic-native';

import { AlertPage } from '../pages/alert/alert';

declare var UserAgent;
declare var window: any;

@Component({
    templateUrl: 'app.html'
})
export class MyApp {
    rootPage = TabsPage;

    constructor(public platform: Platform, public alertCtrl: AlertController, public api: Api, public storage: Storage) {
        platform.ready().then(() => {
            window.open = function (url, windowName, windowFeatures) {
                if(url){
                    if(!url.endsWith('#')){
                        let allowed = /^https?:\/\/(www\.)?kinmunity.com/g
                        if(allowed.test(url)){
                            new InAppBrowser(
                                url, '_blank',
                                'location=yes,hidden=no,clearcache=no,clearsessioncache=no,zoom=no'
                            );
                        }else{
                            window.open2(url, windowName, windowFeatures);
                        }
                    }
                }
            }
            try{
                UserAgent.reset();
                UserAgent.get(function(ua:string) {
                    let last:string = " KinmunityApp";
                    if(!ua.endsWith(last)){
                        UserAgent.set(ua+last);
                    }
                });
            }catch(e){;};
            // Okay, so the platform is ready and our plugins are available.
            // Here you can do any higher level native things you might need.
            StatusBar.styleDefault();
            //StatusBar.backgroundColorByHexString('333333');
            setTimeout(()=>{Splashscreen.hide();},300);
            BackgroundMode.setDefaults({
                'title': 'Kinmunity',
                'text': 'Checking notifications',
                'resume': true,
                'silent': true,
            });
            BackgroundMode.enable();
            LocalNotifications.on('click', (notification, state) => {
                if(notification.id==2 || notification.id==3){
                    let alertBtns = [];
                    if(JSON.parse(notification.data)['page']){
                        alertBtns.push({
                            text: 'App',
                            handler: ()=>{
                                this.nav.push(this.alertGetPage(JSON.parse(notification.data)['page']));
                            }
                        });
                    }
                    alertBtns.push({
                        text: 'Site',
                        handler: ()=>{
                            new InAppBrowser(
                                JSON.parse(notification.data)['url'],'_blank',
                                'location=yes,hidden=no,clearcache=no,clearsessioncache=no,zoom=no'
                            );
                        }
                    });
                    alertBtns.push("Dismiss");
                    let alert = this.alertCtrl.create({
                        title: "There's some activity that you haven't seen",
                        subTitle: notification.text,
                        buttons: alertBtns
                    });
                    alert.present();
                }
            });
            this.simulatePushNotifications();
        });
    }

    @ViewChild(Nav) nav: Nav;

    pages: any[] = [
        { title: 'Forums', component: TabsPage },
    ]

    openPage(page){
        this.nav.setRoot(page.component);
    }
    public notificationUnreadAlerts:number = 0;
    public notificationUnreadConversations:number = 0;
    public simulatePushNotifications() {
        let dismissable = setTimeout(()=>{this.simulatePushNotifications();},40*1000);
        try{
            this.storage.ready().then(()=>{
                this.storage.get('accountToken').then((accountToken)=>{
                    if(!accountToken){
                        clearTimeout(dismissable);
                        setTimeout(()=>{this.simulatePushNotifications();},2*1000);
                        return;
                    }
                    let apiRq = this.api.get('MyDataMini',{
                        'cookie':accountToken,
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        if(received['user_id']==0){
                            return;
                        }
                        let alertsUnread = received['alerts_unread'];
                        let conversationsUnread = received['conversations_unread'];
                        let username = received['username'];
                        if(this.notificationUnreadAlerts != alertsUnread){
                            if(alertsUnread){
                                LocalNotifications.schedule({
                                    id:2,
                                    title: 'Kinmunity: Otherkin Community',
                                    text: username+', you have '+alertsUnread+' unviewed alerts',
                                    data: JSON.stringify({
                                        url: 'https://www.kinmunity.com/account/alerts',
                                        page: 'AlertPage',
                                    }),
                                    // icon: 'res://logo',
                                    // smallIcon: 'res://logo',
                                    sound: null,
                                });
                            }else{
                                LocalNotifications.clear(2);
                            }
                            this.notificationUnreadAlerts = alertsUnread;
                        }
                        if(this.notificationUnreadConversations != conversationsUnread){
                            if(conversationsUnread){
                                LocalNotifications.schedule({
                                    id:3,
                                    title: 'Kinmunity: Otherkin Community',
                                    text: username+', you have '+conversationsUnread+' unread conversations',
                                    data: JSON.stringify({
                                        url: 'https://www.kinmunity.com/conversations/',
                                        page: null,
                                    }),
                                    // icon: 'res://logo.png',
                                    // smallIcon: 'res://logo.png',
                                    sound: null,
                                });
                            }else{
                                LocalNotifications.clear(3);
                            }
                            this.notificationUnreadConversations = conversationsUnread;
                        }
                    });
                });
            });
        }catch(e){;};
    }
    public alertGetPage(page:string=""){
        switch(page){
            case 'AlertPage':
            return AlertPage
            default:
            return null;
        }
    }
}
