import { Component } from '@angular/core';

import { NavController, ToastController } from 'ionic-angular';

import { BrowserloginPage } from '../browserlogin/browserlogin';
import { BrowserTermsPage } from '../browserTerms/browserTerms';
import { AlertPage } from '../alert/alert';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

declare var window: any;

@Component({
    selector: 'page-account',
    templateUrl: 'account.html'
})
export class AccountPage {
    account: {token: string, logged: boolean} = {
        token: '',
        logged: false
    };
    user = {};

    constructor(
        public navCtrl: NavController,
        public toastCtrl: ToastController,
        public storage: Storage,
        public api: Api
    )
    {
        this.doRefresh();
    }
    doRefresh() {
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((val)=>{
                this.account.token = val;
                this.account.logged = (val!==null) && ((String(val).length)>0);
            });
        });
        let cacheKey = 'accountFromServer';
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.storage.get(cacheKey).then((cachedAccount)=>{
                    if(!cachedAccount){
                        cachedAccount = {};
                    }else{
                        cachedAccount = JSON.parse(cachedAccount);
                    }
                    this.user = cachedAccount;
                    let apiRq = this.api.get('MyDataMini',{
                        'cookie':accountToken,
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        this.user = received;
                        this.storage.set(cacheKey,JSON.stringify(this.user));
                        console.info(received);
                    });
                });
            });
        });
    }
    doLogin() {
        let toast1 = this.toastCtrl.create({
            message: "Redirecting you to site...",
            duration: 1.5*1000,
            position: 'bottom'
        });
        toast1.present();
        this.navCtrl.push(BrowserloginPage);
    }
    doLogout(){
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                let apiRq = this.api.get('Logout',{
                    'cookie':accountToken,
                });
                apiRq.subscribe((apiReply)=>{
                    if(apiReply.text()!=='null'){
                        this.storage.remove('accountToken');
                        let toast1 = this.toastCtrl.create({
                            message: "Logged out",
                            duration: 1.0*1000,
                            position: 'top'
                        });
                        toast1.present();
                        this.doRefresh();
                    }
                });
            });
        });
    }
    doNuke(){
        this.storage.clear();
        this.account.token = null;
        this.account.logged = false;
        this.user = {};
    }
    ionViewDidEnter(){
        this.doRefresh();
    }
    doOpenTerms(){
        let toast1 = this.toastCtrl.create({
            message: "Redirecting you to site...",
            duration: 1.5*1000,
            position: 'bottom'
        });
        toast1.present();
        this.navCtrl.push(BrowserTermsPage);
    }
    doOpenAlerts(){
        let toast1 = this.toastCtrl.create({
            message: "Redirecting you to site...",
            duration: 1.5*1000,
            position: 'bottom'
        });
        toast1.present();
        this.navCtrl.push(AlertPage);
    }
    doOpenPMs(){
        window.open('https://www.kinmunity.com/conversations/');
    }

    clickRegister(){
        window.open('https://www.kinmunity.com/register');
    }

    doPersonalDetails(){
        window.open('https://www.kinmunity.com/account/personal-details');
    }

    doOpenPreferences(){
        window.open('https://www.kinmunity.com/account/preferences');
    }
}
