import { Component } from '@angular/core';

import { NavController, ToastController } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

import { ThreadPage } from '../thread/thread';

declare var window: any;

@Component({
    selector: 'page-alert',
    templateUrl: 'alert.html'
})
export class AlertPage {
    alerts = [];

    constructor(public navCtrl: NavController, public api: Api, public storage: Storage, public toastCtrl: ToastController) {
        this.doUpdate();
    }

    doUpdate(){
        let cacheKey = 'alerts';
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.storage.get(cacheKey).then((cachedAlerts)=>{
                    if(!cachedAlerts){
                        cachedAlerts = [];
                    }else{
                        try{
                            cachedAlerts = JSON.parse(cachedAlerts);
                        }catch(e){;}
                    }
                    this.alerts = cachedAlerts;
                    let apiRq = this.api.get('AlertList',{
                        'cookie':accountToken,
                    });
                    apiRq.subscribe((apiReply)=>{
                        if(apiReply['_body'].length==0){
                            console.log('Empty reply')
                            return
                        }
                        let received = apiReply.json();
                        let alerts = [];
                        for(let threadId in received){
                            alerts.push(received[threadId]);
                        }
                        alerts.reverse();
                        this.storage.set(cacheKey,JSON.stringify(alerts));
                        this.alerts = alerts;
                        console.info(received);
                        console.info(alerts);
                    });
                });
            });
        });
    }
    handleAlert(alert){
        switch(alert['content_type']){
            case 'post':
            this.navCtrl.push(ThreadPage,{
                'clickedThread': alert['content'],
            });
            break;

            case 'thread':
            this.navCtrl.push(ThreadPage,{
                'clickedThread': alert['content'],
            });
            break;

            default:
            let toast1 = this.toastCtrl.create({
                message: "Unknown type: '"+alert['content_type']+"'",
                duration: 1.0*1000,
                position: 'top'
            });
            toast1.present();
            break;
        }
    }
    markRead(){
        this.storage.get('accountToken').then((accountToken)=>{
            if(!accountToken){
                accountToken = "";
            }
            let apiRq = this.api.get('AlertMarkRead',{
                'cookie':accountToken,
            });
            apiRq.subscribe((apiReply)=>{
                this.doUpdate();
            });
        });
    }

    openInBrowser(){
        window.open('https://www.kinmunity.com/account/alerts');
    }
}
