import { Component, ViewChild } from '@angular/core';

import { NavController, ToastController, AlertController, Content } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

import { BackgroundMode } from 'ionic-native';

import { BrowserloginPage } from '../browserlogin/browserlogin';
import { ChatOnlinePage } from '../chatOnline/chatOnline';

declare var window: any;

@Component({
    selector: 'page-chat',
    templateUrl: 'chat.html'
})
export class ChatPage {
    public roomSelectOptions = {
        title: 'Select room',
        subTitle: 'Select the room you want to join'
    };
    @ViewChild(Content) content: Content;
    public message:string = '';
    public loggedIn:boolean = false;
    public canSee:boolean = false;
    public canSay:boolean = false;
    public onlines:number = 0;
    public online:Array<Object> = [];
    public roomsList:Array<number> = [-1];
    public selectedRoom:Object;
    public rooms:Array<Object>;
    public roomsDefault:Array<Object> = [
        {
            'room_id':-1,
            'room_name':'Loading...',
            'room_description':'Please wait',
            'room_speakable': false
        }
    ];
    public messagelatest:number = 0;
    public messagesids:Array<number> = [];
    public messages:Object = {};
    public visiblemessages:Array<Object> = [];
    constructor(public navCtrl: NavController, public api: Api, public storage: Storage, public toastCtrl: ToastController, public alertCtrl: AlertController) {
        this.rooms = this.roomsDefault;
        this.selectedRoom=this.rooms[0];
        this.canSay = this.rooms['room_speakable'];
        this.doUpdateMessages();
        this.doOnline();
        this.doUpdate();
        try{
            BackgroundMode.onactivate().subscribe(()=>{
                console.log('BackgroundMode.onactivate()');
                this.windowActive = false;
            });
            BackgroundMode.ondeactivate().subscribe(()=>{
                console.log('BackgroundMode.ondeactivate()');
                this.windowActive = true;
            });
        }catch(e){;};
    }
    public windowActive = true;
    ionViewDidEnter(){
        setTimeout(()=>{this.content.scrollToBottom();},250);
    }
    public roomsDataSaver:number = 0;
    public permsDataSaver:number = 0;
    doUpdate(){
        if(!this.windowActive){
            setTimeout(()=>{this.doUpdate();},2000);
            return;
        }
        console.log('CHAT: doUpdate()');
        //this.api.post('',{'asd':'jkl','zxc':'mnb'}).subscribe(r=>console.log(r.json()));
        let cacheRoomsKey = 'chat-rooms';
        let cachePermsKey = 'chat-perms';
        setTimeout(()=>{this.doUpdate();},3500);
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.loggedIn = accountToken!="";
                let permsT = setTimeout(()=>{
                    this.storage.get(cachePermsKey).then((cachedPerms)=>{
                        if(cachedPerms){
                            cachedPerms = JSON.parse(cachedPerms);
                            this.canSee = cachedPerms['see'];
                            this.canSay = cachedPerms['say'];
                        }
                        let apiRq = this.api.get('ChatPermissions',{
                            'cookie':accountToken,
                        });
                        apiRq.subscribe((apiReplyPermissons)=>{
                            let receivedPerms = apiReplyPermissons.json();
                            this.canSee = receivedPerms['see'];
                            this.canSay = receivedPerms['say'];
                            this.storage.set(cachePermsKey,JSON.stringify(receivedPerms));
                        });
                    });
                },500);
                let roomsT = setTimeout(()=>{
                    this.storage.get(cacheRoomsKey).then((cachedRooms)=>{
                        if(cachedRooms){
                            let parsed = JSON.parse(cachedRooms);
                            this.roomsList = parsed[0];
                            this.rooms = parsed[1];
                            if(this.selectedRoom['room_id']==-1){
                                this.selectedRoom = parsed[1][0];
                            }
                        }
                        let apiRq = this.api.get('ChatListRooms',{
                            'cookie':accountToken,
                        });
                        apiRq.subscribe((apiReplyRoomList)=>{
                            let receivedRoomList = apiReplyRoomList.json();
                            if(receivedRoomList === null){
                                this.rooms = this.roomsDefault;
                                this.roomsList = [-1];
                                this.selectedRoom = this.roomsDefault[0];
                                return;
                            }
                            this.roomsList = receivedRoomList[0];
                            this.rooms = receivedRoomList[1];
                            if(this.selectedRoom['room_id']==-1){
                                this.selectedRoom = this.rooms[0];
                            }
                            this.storage.set(cacheRoomsKey,JSON.stringify(receivedRoomList));
                        });
                    });
                },1000);
                if(!this.loggedIn){
                    this.rooms = this.roomsDefault;
                    this.roomsList = [-1];
                    this.selectedRoom = this.roomsDefault[0];
                    this.canSee = false;
                    this.canSay = false;
                    clearTimeout(roomsT);
                    clearTimeout(permsT);
                    return;
                }
                if(
                    this.selectedRoom['room_id']!=-1 //room list loaded
                    &&
                    this.roomsDataSaver%7!=0 //not in time slice
                )
                {clearTimeout(roomsT);} //cancel delayed call
                this.roomsDataSaver+=1;
                if(
                    this.selectedRoom['room_id']!=-1 //room list loaded
                    &&
                    this.permsDataSaver%5!=0 //not in time slice
                ){clearTimeout(permsT);} //cancel delayed call
                this.permsDataSaver+=1;
            });
        });
    }
    timeoutUpdate = null;
    doUpdateMessages(){
        if(!this.windowActive){
            setTimeout(()=>{this.doUpdateMessages();},2000);
            return;
        }
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.loggedIn = accountToken!="";
                if(!this.loggedIn || !this.canSee){
                    let callMeAgain = ()=>{this.doUpdateMessages();};
                    setTimeout(callMeAgain,1000);
                    return;
                }
                let apiRq = this.api.get('ChatRoomListMessages',{
                    'cookie':accountToken,
                    'messagelatest':this.messagelatest,
                    'pollingtime':25,
                });
                this.api.setTimeout(apiRq,30000,()=>{this.doUpdateMessages();});
                apiRq.subscribe((apiReply)=>{
                    let callMeAgain = ()=>{this.doUpdateMessages();};
                    let cancellable = setTimeout(callMeAgain,10);
                    let received = apiReply.json();
                    if(received===null){ //not logged in... cancel last request and wait a bit more
                        clearTimeout(cancellable);
                        setTimeout(callMeAgain, 3000);
                        return;
                    }
                    let scroll = false;
                    for(let roomid in received){
                        if(!(roomid in this.messages)){
                            this.messages[roomid] = [];
                        }
                        let messages = received[roomid];
                        for(let messageid in messages){
                            if(!(Number(messageid) in this.messagesids)){
                                this.messages[roomid].push(messages[messageid]);
                                this.visiblemessages.push(messages[messageid]);
                                this.messagesids.push(Number(messageid));
                                scroll = true;
                            }
                            this.messagelatest = Math.max(this.messagelatest,Number(messageid));
                        }
                        let cmp = (a,b)=>{
                            return a['message_id']-b['message_id'];
                        };
                        this.messages[roomid].sort(cmp);
                        this.visiblemessages.sort(cmp);
                    }
                    if(scroll){
                        setTimeout(()=>{this.content.scrollToBottom();},50);
                        setTimeout(()=>{this.content.scrollToBottom();},100);
                        setTimeout(()=>{this.content.scrollToBottom();},250);
                    }
                    console.log('CHAT: Long polling results: ',received, ', ',this.messagelatest);
                },(error)=>{
                    let callMeAgain = ()=>{this.doUpdateMessages();};
                    setTimeout(callMeAgain,2500);
                });
            });
        });
    }
    timeoutOnline = null;
    doOnline() {
        if(!this.windowActive){
            setTimeout(()=>{this.doOnline();},2000);
            return;
        }
        if(this.timeoutOnline){
            clearTimeout(this.timeoutOnline);
        }
        this.timeoutOnline = setTimeout(()=>{this.doOnline();}, 15*1000);
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.loggedIn = accountToken!="";
                let apiRq = this.api.get('ChatRoomOnlineGet',{
                    'cookie':accountToken,
                    'roomid':this.selectedRoom['room_id'],
                });
                apiRq.subscribe((apiReply)=>{
                    let received = apiReply.json();
                    this.online = received;
                    if(this.online!==null){
                        this.onlines = this.online.length;
                    }
                });
            });
        });
    }
    lastSent:string = null;
    doSend(){
        if(this.lastSent === this.message){
            let toast1 = this.toastCtrl.create({
                message: "Won't send the same message twice",
                duration: 2.5*1000,
                position: 'bottom'
            });
            toast1.present();
            return
        }
        console.log('CHAT: doSend()');
        if(this.loggedIn && this.canSee && this.canSay && this.selectedRoom['room_speakable']){
            if(this.message.length){
                this.storage.ready().then(()=>{
                    this.storage.get('accountToken').then((accountToken)=>{
                        if(!accountToken){
                            accountToken = "";
                        }
                        if(accountToken.length){
                            let apiRq = this.api.get('ChatRoomMessageSend',{
                                'cookie':accountToken,
                                'message':this.message,
                                'roomid':this.selectedRoom['room_id']
                            });
                            apiRq.subscribe((apiReply)=>{
                                let response = apiReply.json();
                                console.log(response);
                                if(response[0]){
                                    this.message = "";
                                    this.lastSent = null;
                                    //this.doUpdateMessages(true);
                                }else{
                                    let toast1 = this.toastCtrl.create({
                                        message: response[1],
                                        duration: 2.0*1000,
                                        position: 'bottom'
                                    });
                                    toast1.present();
                                }
                            },(error)=>{
                                let toast1 = this.toastCtrl.create({
                                    message: 'Some error happened',
                                    duration: 2.0*1000,
                                    position: 'bottom'
                                });
                                toast1.present();
                                this.lastSent = null;
                            });
                        }else{
                            let toast1 = this.toastCtrl.create({
                                message: "Not logged in",
                                duration: 2.5*1000,
                                position: 'bottom'
                            });
                            toast1.present();
                        }
                    });
                });
            }else{
                let toast1 = this.toastCtrl.create({
                    message: "Cannot send empty messages",
                    duration: 2.5*1000,
                    position: 'bottom'
                });
                toast1.present();
            }
        }else{
            let toast1 = this.toastCtrl.create({
                message: "Insufficient permissions",
                duration: 2.5*1000,
                position: 'bottom'
            });
            toast1.present();
        }
    }
    detectSend(keyCode){
        if(keyCode === 13){
            this.doSend();
        }
    }
    doOnlineBtn() {
        if(this.onlines>0){
            this.navCtrl.push(ChatOnlinePage,{'online':this.online,'room':this.selectedRoom});
        }else{
            let alert = this.alertCtrl.create({
                title: 'No one here',
                subTitle: "This means you're alone",
                buttons: ['Cry']
            });
            alert.present();
        }
    }
    doLogin() {
        this.navCtrl.push(BrowserloginPage);
    }
    doChange(){
        //console.log(this.selectedRoom);
        setTimeout(()=>{this.content.scrollToBottom();},250);
    }
    doRoomDescriptionBtn(){
        let alert = this.alertCtrl.create({
            title: this.selectedRoom['room_name'],
            subTitle: this.selectedRoom['room_description'],
            buttons: ['Dismiss']
        });
        alert.present();
    }
    doRoomSelectBtn(){
        if(this.canSee){
            let alertInputs:Array<Object> = [];
            for(let room of this.rooms){
                alertInputs.push({
                    type:'radio',
                    name:room['room_name'],
                    label:room['room_name'],
                    id:JSON.stringify(room),
                    value:JSON.stringify(room),
                    checked:this.selectedRoom['room_id']===room['room_id'],
                });
            }
            let alert = this.alertCtrl.create(
                {
                    title: this.roomSelectOptions['title'],
                    subTitle: this.roomSelectOptions['subTitle'],
                    inputs:alertInputs,
                    buttons: [
                        {
                            text: 'Cancel',
                            role: 'cancel',
                        },
                        {
                            text: 'OK',
                            role: 'ok',
                            handler: (data) => {
                                this.selectedRoom = JSON.parse(data);
                            }
                        }
                    ]
                }
            );
            alert.present();
        }
    }

    openInBrowser(){
        window.open('https://www.kinmunity.com/chat/');
    }
}
