import { Component } from '@angular/core';

import { NavController, NavParams } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

import { ThreadPage } from '../thread/thread';
import { ThreadNewPage } from '../threadNew/threadNew';

declare var window: any;

@Component({
    selector: 'page-forum',
    templateUrl: 'forum.html'
})
export class ForumPage {
    private forum = {};
    private page = 1;
    private pages = 1;
    private threads = [];
    public loggedIn:boolean = false;
    public canReply:boolean = false;

    constructor(public navCtrl: NavController, public navParams: NavParams, public api: Api, public storage: Storage) {
        this.forum = this.navParams.get('clickedForum');
        if(this.forum['ct']){
            this.navCtrl.pop();
            return;
        }
        this.doUpdate();
        let cacheKey = 'forum-'+this.forum['id']+'-permReply';
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.storage.get(cacheKey).then((cachedPerm)=>{
                    if(!cachedPerm){
                        cachedPerm = 0;
                    }else{
                        cachedPerm = JSON.parse(cachedPerm);
                    }
                    this.canReply = cachedPerm;
                    let apiRq = this.api.get('ForumInsertThread',{
                        'cookie':accountToken,
                        'frm':this.forum['id'],
                        'permtst':1
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        this.canReply = received[0];
                        this.storage.set(cacheKey,JSON.stringify(this.canReply));
                    });
                });
            });
        });
    }

    doUpdate(){
        let cacheKey = 'forum-'+this.forum['id']+'-'+this.page;
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.loggedIn = accountToken!="";
                this.storage.get(cacheKey).then((cachedForum)=>{
                    if(!cachedForum){
                        cachedForum = [];
                    }else{
                        cachedForum = JSON.parse(cachedForum);
                    }
                    this.threads = cachedForum;
                    let apiRq = this.api.get('ForumListThread',{
                        'cookie':accountToken,
                        'frm':this.forum['id'],
                        'pag':this.page
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        let threads = [];
                        for(let threadId in received['threads']){
                            threads.push(received['threads'][threadId]);
                        }
                        threads.reverse();
                        this.storage.set(cacheKey,JSON.stringify(threads));
                        this.threads = threads;
                        console.info(received);
                        console.info(threads);
                        this.page = received['page'];
                        this.pages = received['pages'];
                    });
                });
            });
        });
    }
    enterThread(thread){
        this.navCtrl.push(ThreadPage,{
            'clickedThread': thread,
        });
    }

    doPageP(){
        if(this.page<this.pages){
            this.page+=1;
        }
        this.doUpdate();
    }

    doPageM(){
        if(this.page>1){
            this.page-=1;
        }
        this.doUpdate();
    }

    doReply(){
        if(this.canReply){
            this.navCtrl.push(ThreadNewPage,{
                'clickedForum':this.forum
            });
        }
    }
    openInBrowser(){
        window.open('https://www.kinmunity.com/forums/'+this.forum['id']);
    }
}
