import { Component, ViewChild } from '@angular/core';

import { NavController, NavParams, Content } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

import { ThreadReplyPage } from '../threadReply/threadReply';

declare var window: any;

@Component({
    selector: 'page-thread',
    templateUrl: 'thread.html'
})
export class ThreadPage {
    private forum = {};
    private thread = {};
    private posts = [];
    private firstUnreadPost = 0;
    public loggedIn:boolean = false;
    public canReply:boolean = false;

    constructor(public navCtrl: NavController, public navParams: NavParams, public api: Api, public storage: Storage) {
        this.thread = this.navParams.get('clickedThread');
        this.doUpdate();
        let cacheKey = 'thread-'+this.thread['thread_id']+'-permReply';
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.storage.get(cacheKey).then((cachedPerm)=>{
                    if(!cachedPerm){
                        cachedPerm = 0;
                    }else{
                        cachedPerm = JSON.parse(cachedPerm);
                    }
                    this.canReply = cachedPerm;
                    let apiRq = this.api.get('ForumThreadInsertPost',{
                        'cookie':accountToken,
                        'thd':this.thread['thread_id'],
                        'permtst':1
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        this.canReply = received[0];
                        this.storage.set(cacheKey,JSON.stringify(this.canReply));
                        console.log(received);
                    });
                });
            });
        });
    }

    doUpdate(){
        let cacheKey = 'thread-'+this.thread['thread_id'];
        this.storage.ready().then(()=>{
            this.storage.get('accountToken').then((accountToken)=>{
                if(!accountToken){
                    accountToken = "";
                }
                this.loggedIn = accountToken!="";
                this.storage.get(cacheKey).then((cachedThread)=>{
                    if(!cachedThread){
                        cachedThread = {
                            "forum":{},
                            "thread":{},
                            "firstUnreadPost":0,
                            "posts":[],
                        };
                    }else{
                        cachedThread = JSON.parse(cachedThread);
                    }
                    this.forum = cachedThread['forum'];
                    this.firstUnreadPost = cachedThread['firstUnreadPost'];
                    this.posts = cachedThread['posts'];
                    if(this.firstUnreadPost!=0){
                        setTimeout(this.scrollToEnd,100);
                    }
                    let apiRq = this.api.get('ForumThreadListPost',{
                        'cookie':accountToken,
                        'thd':this.thread['thread_id'],
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        let posts = [];
                        for(let postId in received['posts']){
                            posts.push(received['posts'][postId]);
                        }
                        this.posts = posts;
                        received['posts']=posts;
                        this.storage.set(cacheKey,JSON.stringify(received));
                        console.info(received);
                        console.info(posts);
                        this.forum = received['forum'];
                        this.firstUnreadPost = received['firstUnreadPost'];
                        if(this.firstUnreadPost!=0){
                            setTimeout(this.scrollToEnd,100);
                        }
                    });
                });
            });
        });
    }
    doReply(thread,post){
        if(this.canReply){
            this.navCtrl.push(ThreadReplyPage,{
                'clickedThread': thread,
                'clickedPost': post,
            });
        }
    }

    ngAfterViewInit(){
        this.scrollableToEnd = true;
        this.ionViewDidEnter();
    }
    ionViewDidEnter(){
        if(this.scrollToEndDelayed){
            this.scrollToEndDelayed = false;
            this.scrollToEnd();
        }
        if(this.firstUnreadPost!=0){
            this.scrollToEnd();
        }
    }
    scrollableToEnd:boolean = false;
    scrollToEndDelayed:boolean = false;
    @ViewChild(Content) content: Content;
    scrollToEnd(){
        if(this.scrollableToEnd){
            this.content.scrollToBottom();
            this.scrollToEndDelayed = false;
        }else{
            this.scrollToEndDelayed = true;
            setTimeout(this.scrollToEnd,50);
        }
    }
    openInBrowser(){
        window.open('https://www.kinmunity.com/threads/'+this.thread['thread_id']);
    }
}
