import { Component } from '@angular/core';

import { NavController, NavParams, ToastController } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

@Component({
    selector: 'page-threadNew',
    templateUrl: 'threadNew.html'
})
export class ThreadNewPage {
    public content = "";
    public title = "";
    public forum:Object = {};

    constructor(public navCtrl: NavController, public navParams: NavParams, public api: Api, public storage: Storage, public toastCtrl: ToastController) {
        this.forum = this.navParams.get('clickedForum');
        this.draftLoad();
    }
    draftLoad(){
        let draftKey = 'forum-'+this.forum['id']+'-draft';
        this.storage.ready().then(()=>{
            this.storage.get(draftKey+'t').then((draft)=>{
                this.title = draft;
            });
            this.storage.get(draftKey+'c').then((draft)=>{
                this.content = draft;
            });
        });
    }
    draftSave(){
        let draftKey = 'forum-'+this.forum['id']+'-draft';
        this.storage.ready().then(()=>{
            this.storage.set(draftKey+'t',this.title);
        });
        this.storage.ready().then(()=>{
            this.storage.set(draftKey+'c',this.content);
        });
    }
    draftNuke(){
        let draftKey = 'forum-'+this.forum['id']+'-draft';
        this.storage.remove(draftKey+'t');
        this.storage.remove(draftKey+'c');
        this.content = "";
        this.title = "";
    }
    doSend(){
        if(this.content && this.title){
            this.storage.ready().then(()=>{
                this.storage.get('accountToken').then((accountToken)=>{
                    if(!accountToken){
                        accountToken = "";
                    }
                    let apiRq = this.api.get('ForumInsertThread',{
                        'cookie':accountToken,
                        'frm':this.forum['id'],
                        'message':this.content,
                        'title':this.title,
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        let toast1 = this.toastCtrl.create({
                            message: received[1],
                            duration: 2.5*1000,
                            position: 'top'
                        });
                        let toast2 = this.toastCtrl.create({
                            message: received[2],
                            duration: 5.0*1000,
                            position: 'bottom'
                        });
                        toast1.present();
                        toast2.present();
                        if(received[0]){
                            this.draftNuke();
                            this.navCtrl.pop();
                        }
                    });
                });
            });
        } else {
            if(!Boolean(this.content)){
                let toast1 = this.toastCtrl.create({
                    message: "Cannot create empty threads",
                    duration: 2.5*1000,
                    position: 'top'
                });
                toast1.present();
            }
            if(!Boolean(this.title)){
                let toast2 = this.toastCtrl.create({
                    message: "Cannot create untitled threads",
                    duration: 2.5*1000,
                    position: 'top'
                });
                toast2.present();
            }
        }
    }
    onInput(ev){
        //this.content = ev.target.value;
        this.draftSave();
    }
    ionViewWillLeave(){
        this.draftSave();
    }
}
