import { Component } from '@angular/core';

import { NavController, NavParams, ToastController } from 'ionic-angular';

import { Api } from '../../providers/api';
import { Storage } from '@ionic/storage';

@Component({
    selector: 'page-threadReply',
    templateUrl: 'threadReply.html',
})
export class ThreadReplyPage {
    public content:string = "";
    public thread:Object = {};
    public post:Object = {};
    constructor(public navCtrl: NavController, public navParams: NavParams, public api: Api, public storage: Storage, public toastCtrl: ToastController) {
        this.thread = this.navParams.get('clickedThread');
        this.post = this.navParams.get('clickedPost');
        if(this.post){
            console.log(this.thread);
            this.content = '[QUOTE="'+this.post['username']+', post: '+this.post['post_id']+', member: '+this.post['user_id']+'"]'+this.post['message']+'[/QUOTE]\n';
            this.draftSave();
        }else{
            this.draftLoad();
        }
    }
    draftLoad(){
        let draftKey = 'thread-'+this.thread['thread_id']+'-draft';
        this.storage.ready().then(()=>{
            this.storage.get(draftKey).then((draft)=>{
                this.content = draft;
            });
        });
    }
    draftSave(){
        let draftKey = 'thread-'+this.thread['thread_id']+'-draft';
        this.storage.ready().then(()=>{
            this.storage.set(draftKey,this.content);
        });
    }
    draftNuke(){
        let draftKey = 'thread-'+this.thread['thread_id']+'-draft';
        this.storage.remove(draftKey);
        this.content = "";
    }
    doSend(){
        if(this.content){
            this.storage.ready().then(()=>{
                this.storage.get('accountToken').then((accountToken)=>{
                    if(!accountToken){
                        accountToken = "";
                    }
                    let apiRq = this.api.get('ForumThreadInsertPost',{
                        'cookie':accountToken,
                        'thd':this.thread['thread_id'],
                        'message':this.content
                    });
                    apiRq.subscribe((apiReply)=>{
                        let received = apiReply.json();
                        let toast1 = this.toastCtrl.create({
                            message: received[1],
                            duration: 2.5*1000,
                            position: 'top'
                        });
                        let toast2 = this.toastCtrl.create({
                            message: received[2],
                            duration: 5.0*1000,
                            position: 'bottom'
                        });
                        toast1.present();
                        toast2.present();
                        if(received[0]){
                            this.draftNuke();
                            this.navCtrl.pop();
                        }
                    });
                });
            });
        } else {
            let toast1 = this.toastCtrl.create({
                message: "Cannot send empty messages",
                duration: 2.5*1000,
                position: 'top'
            });
            toast1.present();
        }
    }
    onInput(ev){
        this.content = ev.target.value;
        this.draftSave();
    }
    ionViewWillLeave(){
        this.draftSave();
    }
}
