import { Injectable } from '@angular/core';
import { Http, RequestOptions, URLSearchParams, Response, Headers } from '@angular/http';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/timeout';
import { Observable } from 'rxjs/Observable';

/**
* Api is a generic REST Api handler. Set your API url first.
*/
@Injectable()
export class Api {
    url: string;
    //url_root: string = 'http://192.168.0.108/kinmunity.com';
    url_root: string = 'https://www.kinmunity.com';
    url_folder: string = '/mobile-api';

    constructor(public http: Http) {
        this.url = this.url_root + this.url_folder;
    }

    get(endpoint: string, params?: any, options?: RequestOptions) {
        if (!options) {
            options = new RequestOptions();
        }

        // Support easy query params for GET requests
        if (params) {
            let p = new URLSearchParams();
            for(let k in params) {
                p.set(k, params[k]);
            }
            // Set the search field if we have params and don't already have
            // a search field set in options.
            options.search = !options.search && p || options.search;
        }
        return this.http.get(this.url + '/' + endpoint, options);
    }

    post(endpoint: string, body: any) {
        let headers = new Headers();
        headers.append('Content-Type', 'text/plain');
        if(typeof(body) !== 'string'){
            body = JSON.stringify(body);
        }
        let hexEncode = function(str){
            var hex, i;

            var result = "";
            for (i=0; i<str.length; i++) {
                hex = str.charCodeAt(i).toString(16);
                result += ("000"+hex).slice(-4);
            }
            return result
        }
        body = hexEncode(body); //workaround to be able to send brackets in message
        return this.http.post(this.url + '/' + endpoint, body, {headers: headers});
    }
    put(endpoint: string, body: any, options?: RequestOptions) {
        return this.http.put(this.url + '/' + endpoint, body, options);
    }

    delete(endpoint: string, body: any, options?: RequestOptions) {
        return this.http.post(this.url + '/' + endpoint, body, options);
    }

    patch(endpoint: string, body: any, options?: RequestOptions) {
        return this.http.put(this.url + '/' + endpoint, body, options);
    }

    encapsulatePost(params: { [index: string]: any; }){
        let data = new URLSearchParams();
        for(var key in params){
            data.append(key, params[key]);
        }
        return data;
    }

    setTimeout(rq:Observable<Response>,timeout:number,callback){
        return rq.timeout(timeout,callback);
    }
}
